<?php
/**
 * This file is part of the SetaPDF-Core Component
 *
 * @copyright  Copyright (c) 2017 Setasign - Jan Slabon (https://www.setasign.com)
 * @category   SetaPDF
 * @package    SetaPDF_Core
 * @subpackage Type
 * @license    https://www.setasign.com/ Commercial
 * @version    $Id: Array.php 1052 2017-05-18 09:49:46Z jan.slabon $
 */

/**
 * Class representing an array
 *
 * @copyright  Copyright (c) 2017 Setasign - Jan Slabon (https://www.setasign.com)
 * @category   SetaPDF
 * @package    SetaPDF_Core
 * @subpackage Type
 * @license    https://www.setasign.com/ Commercial
 */
class SetaPDF_Core_Type_Array extends SetaPDF_Core_Type_AbstractType
    implements SplObserver, Countable, ArrayAccess, RecursiveIterator
{
    /**
     * The values
     *
     * An array of {@link SetaPDF_Core_Type_AbstractType} objects
     *
     * @var $_values array
     */
    protected $_values = array();

    /**
     * The array count
     *
     * @var integer
     */
    protected $_count = 0;

    /** @noinspection PhpMissingParentConstructorInspection */
    /**
     * The constructor.
     *
     * @param array $values An array filled with values of type SetaPDF_Core_Type_AbstractType
     * @throws InvalidArgumentException
     */
    public function __construct(array $values = null)
    {
        unset($this->_observed);
        if (null !== $values) {
            foreach ($values AS $value) {
                if (!($value instanceof SetaPDF_Core_Type_AbstractType)) {
                    throw new InvalidArgumentException('Parameter should be a values of type SetaPDF_Core_Type_AbstractType');
                }

                if ($value instanceof SetaPDF_Core_Type_IndirectObject) {
                    $value = new SetaPDF_Core_Type_IndirectReference($value);
                }

                $this->_values[] = $value;
                $this->_count++;
            }
        }
    }

    /**
     * Clone the object recursively in the context of a document.
     *
     * @param SetaPDF_Core_Document $document
     * @return SetaPDF_Core_Type_AbstractType
     */
    public function deepClone(SetaPDF_Core_Document $document)
    {

    }

    /**
     * Add an observer to the object.
     *
     * This method forwards the attach()-call
     * to all values of this array.
     *
     * @param SplObserver $observer
     */
    public function attach(SplObserver $observer)
    {

    }

    /**
     * Triggered if a value of this object is changed.
     *
     * Forward this to other observers.
     *
     * @param SplSubject $SplSubject
     */
    public function update(SplSubject $SplSubject)
    {

    }

    /**
     * Sets the values.
     *
     * @param array|SetaPDF_Core_Type_Array $values An array of SetaPDF_Core_Type_AbstractType objects
     * @throws InvalidArgumentException
     */
    public function setValue($values)
    {

    }

    /**
     * Gets the value.
     *
     * @return array
     */
    public function getValue()
    {
        return $this->_values;
    }

    /**
     * Returns the type as a formatted PDF string.
     *
     * @param SetaPDF_Core_Document $pdfDocument
     * @return string
     */
    public function toPdfString(SetaPDF_Core_Document $pdfDocument)
    {

    }

    /**
     * Writes the type as a formatted PDF string to the document.
     *
     * @param SetaPDF_Core_Document $pdfDocument
     */
    public function writeTo(SetaPDF_Core_Document $pdfDocument)
    {

    }

    /**
     * Release objects/memory.
     *
     * @see SetaPDF_Core_Type_AbstractType::cleanUp()
     */
    public function cleanUp()
    {
        if (!isset($this->_observed)) {
            $this->clear();
        }
    }

    /**
     * Converts the PDF data type to a PHP data type and returns it.
     *
     * @see SetaPDF_Core_Type_AbstractType::toPhp()
     * @return array
     */
    public function toPhp()
    {
        $result = array();

        foreach ($this->_values AS $value) {
            $result[] = $value->toPhp();
        }

        return $result;
    }

    /**
     * Returns the number of elements in the array.
     *
     * @link http://www.php.net/Countable.count Countable::count
     * @return int
     */
    public function count()
    {
        return $this->_count;
    }

    /**
     * Checks whether a offset exists.
     *
     * @link http://www.php.net/ArrayAccess.offsetExists ArrayAccess::offsetExists
     * @param int $offset An offset to check for.
     * @return boolean
     */
    public function offsetExists($offset)
    {
        return isset($this->_values[$offset]);
    }

    /**
     * Offset to retrieve.
     *
     * @link http://www.php.net/ArrayAccess.offsetGet ArrayAccess::offsetGet
     * @param int $offset The offset to retrieve.
     * @return SetaPDF_Core_Type_AbstractType|null
     */
    public function offsetGet($offset)
    {
        return isset($this->_values[$offset])
            ? $this->_values[$offset]
            : null;
    }

    /**
     * Offset to set.
     *
     * @link http://www.php.net/ArrayAccess.offsetSet ArrayAccess::offsetSet
     * @param null|int $offset The offset to assign the value to.
     * @param SetaPDF_Core_Type_AbstractType $value The value to set.
     * @throws InvalidArgumentException
     */
    public function offsetSet($offset, $value)
    {

    }

    /**
     * Checks whether a offset exists.
     *
     * @link http://www.php.net/ArrayAccess.offsetUnset ArrayAccess::offsetUnset
     * @param string $offset
     */
    public function offsetUnset($offset)
    {

    }

    /**
     * Prepends one element to the beginning of the array.
     *
     * @param SetaPDF_Core_Type_AbstractType $value
     */
    public function unshift(SetaPDF_Core_Type_AbstractType $value)
    {

    }

    /**
     * Inserts an element before another one.
     *
     * Index mustn't be higher than the count of elements in array.
     *
     * Index 0 is allowed in an empty array.
     *
     * @param SetaPDF_Core_Type_AbstractType $value
     * @param null|integer $beforeIndex
     * @throws InvalidArgumentException
     */
    public function insertBefore(SetaPDF_Core_Type_AbstractType $value, $beforeIndex = 0)
    {

    }

    /**
     * Pushes a value onto the end of the array.
     *
     * @param SetaPDF_Core_Type_AbstractType $value
     */
    public function push(SetaPDF_Core_Type_AbstractType $value)
    {
        $this->offsetSet(null, $value);
    }


    /**
     * Returns the index of the element.
     *
     * If the element isn't in this array -1 will returned.
     *
     * @param SetaPDF_Core_Type_AbstractType $element
     * @return int
     */
    public function indexOf(SetaPDF_Core_Type_AbstractType $element)
    {
        
    }

    /**
     * Returns the current element.
     *
     * @link http://www.php.net/Iterator.current Iterator::current
     * @return SetaPDF_Core_Type_AbstractType
     */
    public function current()
    {
        return current($this->_values);
    }

    /**
     * Moves forward to next element.
     *
     * @link http://www.php.net/Iterator.next Iterator::next
     * @return SetaPDF_Core_Type_AbstractType
     */
    public function next()
    {
        return next($this->_values);
    }

    /**
     * Returns the key of the current element.
     *
     * @link http://www.php.net/Iterator.key Iterator::key
     * @return integer
     */
    public function key()
    {
        return key($this->_values);
    }

    /**
     * Checks if current position is valid.
     *
     * @see http://www.php.net/Iterator.valid
     * @return boolean
     */
    public function valid()
    {
        // TODO: Change to a property which is updated in next()
        return current($this->_values) !== false;
    }

    /**
     * Rewinds the Iterator to the first element.
     *
     * @link http://www.php.net/Iterator.rewind Iterator::rewind
     */
    public function rewind()
    {
        reset($this->_values);
    }

    /**
     * Returns an iterator for the current entry.
     *
     * @link http://www.php.net/RecursiveIterator.getChildren RecursiveIterator::getChildren
     * @return array
     */
    public function getChildren()
    {
        $current = current($this->_values);

        if($current instanceof SetaPDF_Core_Type_AbstractType)
            $current = current($this->_values)->ensure();

        if ($current instanceof SetaPDF_Core_Type_Array)
            return $current;

        return null;
    }

    /**
     * Check whether the current entry is an SetaPDF_Core_Type_Array.
     *
     * @link http://www.php.net/RecursiveIterator.hasChildren RecursiveIterator::hasChildren
     * @return boolean
     */
    public function hasChildren()
    {
        return (current($this->_values)->ensure() instanceof SetaPDF_Core_Type_Array);
    }
}